import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.util.ArrayList;

/**
 * New class TextTest
 * This class is here to test the different methods of the Text class.
 * Author: Roman Schenk, Andrin Fassbind
 * Date: 12.11.2021
 */
public class TextTest {
    Text txt;
    ArrayList<String> stringListe;

    /**
     * This method is here as a preparation for the test.
     */
    @BeforeEach
    void setup() {
        stringListe = new ArrayList<>();
        txt = new Text();
    }

    /**
     * Test Method for add and dummy methods
     */
    @Test
    void addTest() {
        Assertions.assertEquals(0, txt.getText().size());
        Assertions.assertFalse(txt.add(1, "test"));
        Assertions.assertFalse(txt.dummy(1));
        Assertions.assertEquals(0, txt.getText().size());
        Assertions.assertTrue(txt.add("test1"));
        Assertions.assertTrue(txt.add(1, "test2"));
        Assertions.assertEquals("test2", txt.getText().get(0));
        Assertions.assertEquals("test1", txt.getText().get(1));
        Assertions.assertTrue(txt.dummy(2));
        Assertions.assertTrue(txt.dummy());
        Assertions.assertEquals(txt.getText().get(1), txt.getText().get(3));
        Assertions.assertEquals(4, txt.getText().size());
    }

    /**
     * Test Method del
     */
    @Test
    void del() {
        Assertions.assertTrue(txt.add("Das ist der erste Beispiel Text."));
        Assertions.assertTrue(txt.add("Das ist der zweite Beispiel Text."));
        Assertions.assertTrue(txt.add("Das ist der dritte Beispiel Text."));
        Assertions.assertTrue(txt.add("Das ist der vierte Beispiel Text."));
        Assertions.assertFalse(txt.del(5));
        Assertions.assertTrue(txt.del(2));
        Assertions.assertTrue(txt.del());
        Assertions.assertEquals("Das ist der erste Beispiel Text.", txt.getText().get(0));
        Assertions.assertEquals("Das ist der dritte Beispiel Text.", txt.getText().get(1));
    }

    /**
     * Test Method for replace Method
     */
    @Test
    void replace() {
        Assertions.assertTrue(txt.add("Das ist der erste Beispiel Text."));
        Assertions.assertTrue(txt.add("Das ist der zweite Beispiel Text."));
        Assertions.assertFalse(txt.replace(3, "alt", "neu"));
        Assertions.assertTrue(txt.replace(1, "erste", "zweite"));
        Assertions.assertTrue(txt.replace("zweite", "erste"));
        Assertions.assertTrue(txt.add("Text Text hallo Text"));
        Assertions.assertTrue(txt.replace("Text", "Test"));
        Assertions.assertEquals("Das ist der zweite Beispiel Text.", txt.getText().get(0));
        Assertions.assertEquals("Das ist der erste Beispiel Text.", txt.getText().get(1));
        Assertions.assertEquals("Test Test hallo Test", txt.getText().get(2));
    }

    /**
     * Test methode for Index method
     * For TestCase Positiv Testcase One, Three and Four Negativ Testcase Two
     */
    @Test
    void indexTestOne() {
        txt.add("Word word Test");
        txt.add("Word word etc. !!test zweite... Zeile");
        txt.add("Lorem ipsum lorem ipsum   TEST");
        stringListe = txt.index();
        Assertions.assertEquals("Word    1, 2", stringListe.get(0));
        Assertions.assertEquals("Test    1, 2, 3", stringListe.get(1));
    }

    /**
     * Test methode for Index method
     * For TestCase Positiv Testcase Two
     */
    @Test
    void indexTestTwo() {
        txt.add("\"Word word Word Test");
        stringListe = txt.index();
        Assertions.assertEquals("Word    1", stringListe.get(0));
    }

    /**
     * Test methode for Index method
     * For TestCase Negativ Testcase One
     */
    @Test
    void indexTestThree() {
        stringListe = txt.index();
        Assertions.assertEquals(0, stringListe.size());
    }

    /**
     * Test Method to check if index is working with empty Strings
     */
    @Test
    void indexTestFour() {
        txt.add("");
        txt.index();
    }
}
